from typing import Literal

AccountType = Literal["depository", "loan", "mobilemoney"]
Category = Literal["LOANS_AND_CREDIT_CARDS", "BETTING_AND_LOTTERY", "SALARY", "INCOME"]
Currency = Literal["AED", "GHS", "KES", "NGN", "USD", "unknown"]
Impact = Literal["CREDIT", "DEBIT"]
